
###############################################################################################
#  AN EXAMPLE TO FIT THE MULTIDIMENSIONAL RESPONSE MODEL: Classical Inference                 #    
#                                                                                             #
# Data contain 1111 respondent to 21 items of Beck Depression Inventory (BDI)                 #
# Data are described in Fragoso, T. M. and Curi, M. (2013). Improving psychometric assessment #      
# of the Beck Depression Inventory using Multidimensional Item Response Theory. Biometrical   #
# Journal, 55,  527-540                                                                       #
###############################################################################################



# install package ltm, psych, foreign
#install.packages(c("ltm","psych","foreign") 

#1. Read data set

require(foreign)
#wd="C:\\Users\\Jorge Luis\\Copy\\MinicursTRI\\Exemplos\\Multidimensional"
#setwd(wd)
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

bdi=read.spss("bdi Univ - Tengdico.sav")
a=data.frame(bdi)
bdiitems=a[,4:24]


#2. Classical item analysis 
require(ltm)
cronbach.alpha(bdiitems)

require(psych)
alpha(bdiitems)


#3. Evaluation of multidimensionality

fa(bdiitems) #factor analysis using pearson correlation matrix

require(psych)
c=tetrachoric(bdiitems)

fap1 <- fa(r=c$rho)  #factor analysis using tetrachoric correlation matrix
fap1
fap2 <- fa(r=c$rho,nfactors=2,rotate="varimax")
fap2

par(mfrow=c(1,2))
fa.diagram(fap2,cut=0.4)
factor.plot(fap2, cut=0.4,ylim=c(0,1),xlim=c(0,1))
abline(h=0.4,col=4,lty=2)
abline(v=0.4,col=4,lty=2)

###############################################################################
#4. ML Estimation of 2L multidimensional model using ltm: Calibration of items
#with usual parameterization

require(ltm)
#library(compiler)  #files to aceleration of the estimation
#enableJIT(3) 
fit2LM<-ltm(bdiitems ~ z1+z2) #com dois thetas e dois par.a
fit2LM
summary(fit2LM)

detach("package:psych", unload=TRUE) 

#5. ML estimation of abilities with item parameters knowns

fscEAP=factor.scores(fit2LM,method="EAP")
fscEAP

factor.scores(fit2LM, resp.patterns=bdiitems, method = "EAP")
EAP=factor.scores(fit2LM, method = "EAP")$score.dat
write.table(EAP,"EAPBDIac.csv",sep=",") 

#########################################################################################

#7. ML Estimation of 2L multidimensional model using mirt: Calibration of items
#with usual parameterization

require(mirt)
fit2LMmirt<-mirt(bdiitems,2, itemtype ="2PL",verbose =FALSE)
fit2LMmirt
summary(fit2LMmirt)
coef(fit2LMmirt)
coef(fit2LMmirt,rotate="varimax")


#7. ML estimation of abilities with item parameters knowns
scoresest <- fscores(fit2LMmirt, method="EAP", response.pattern =
                       bdiitems)
summary(scoresest)
est_scores <- as.data.frame(scoresest)
summary(est_scores)


# Note. Reviewed your results with carefully  Compare your results with the obtained 
# using other software. For example you can use IRTPRO
# http://www.ssicentral.com/irt/student.html

